﻿using Common.Classes;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace BOF2LevelStatGainEditor
{
    public partial class MainForm : Form
    {
        #region Fields

        private Rom romData;
        private List<Character> characterSets;
        private string selectedRom;
        private bool? justLoadedRom;            // prevents the program to do some things when it loads rom data
        private int previousCharacterIndex;
        private int previousLevelIndex;
        private bool isRetranslatedRom;
        private string[] enemyNames;
        
        #endregion

        #region Constructor

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        #region Event Handlers
        
        /// <summary>
        /// aboutToolStripMenuItem_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox aboutBox = new AboutBox();
            aboutBox.ShowDialog();
        }

        /// <summary>
        /// btnCharacterMaxStats Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCharacterMaxStats_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("You're about to max the stats for the current character. Proceed?",
                Application.ProductName,
                MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                this.romData.MaxStatsForCharacter(cbCharacterList.SelectedIndex);
                MessageBox.Show("Done!", Application.ProductName);
                SetupCharacterLevelFields();
            }
        }

        /// <summary>
        /// btnCharacterMaxStatsAll Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCharacterMaxStatsAll_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("You're about to max the stats for EVERY character in the game. Proceed?",
                Application.ProductName,
                MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                this.romData.MaxStatsForAllCharacters();
                MessageBox.Show("Done!", Application.ProductName);
                SetupCharacterLevelFields();
            }
        }

        /// <summary>
        /// btnCharacterExpNeedAllLevels Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCharacterExpNeedAllLevels_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("You're about to set the experience needed for all levels for the current character in the game. Proceed?",
                Application.ProductName,
                MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                this.romData.SetExperienceNeededAtAllLevelsForCharacter(this.cbCharacterList.SelectedIndex, Convert.ToInt32(numCharacterExp.Value));
                MessageBox.Show("Done!", Application.ProductName);
                SetupCharacterLevelFields();
            }
        }

        /// <summary>
        /// btnAllCharacterExpNeedAllLevels Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAllCharacterExpNeedAllLevels_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("You're about to set the experience needed for all levels for EVERY character in the game. Proceed?",
                Application.ProductName,
                MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                this.romData.SetExperienceNeededAtAllLevelsForAllCharacters(Convert.ToInt32(numCharacterExp.Value));
                MessageBox.Show("Done!", Application.ProductName);
                SetupCharacterLevelFields();
            }
        }

        /// <summary>
        /// cbCharacterList SelectedIndexChanged Event Handler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbCharacterList_SelectedIndexChanged(object sender, EventArgs e)
        {
            int characterIndex = this.cbCharacterList.SelectedIndex;
            if (justLoadedRom.HasValue)
            {
                if (!justLoadedRom.Value)
                {
                    SaveCharacterLevelFields();
                }

                SetupCharacterLevelFields();
            }

            previousCharacterIndex = this.cbCharacterList.SelectedIndex;
        }
        
        /// <summary>
        /// cbLevelList SelectedIndexChanged Event Handler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbLevelList_SelectedIndexChanged(object sender, EventArgs e)
        {
            int levelIndex = this.cbLevelList.SelectedIndex;
            if (justLoadedRom.HasValue)
            {
                if (!justLoadedRom.Value)
                {
                    SaveCharacterLevelFields();
                }

                SetupCharacterLevelFields();
            }

            previousLevelIndex = this.cbLevelList.SelectedIndex;
        }

        /// <summary>
        /// exitToolStripMenuItem Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkInformation_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.gamefaqs.com/snes/563530-breath-of-fire-ii/faqs/5436");
        }

        /// <summary>
        /// MainForm Load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            SetFieldsState(false);
            enemyNames = Monster.LoadEnemyNameFile();
            SetupLevelDropDownList();
        }

        /// <summary>
        /// fileToolStripMenuItem Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                System.Windows.Forms.DialogResult result = this.openFileOpenRom.ShowDialog(this);
                if (result.Equals(System.Windows.Forms.DialogResult.OK))
                {
                    selectedRom = this.openFileOpenRom.FileName;
                    this.Text = "BoF 2 Level Stat Gain Editor - " + selectedRom;
                    this.romData = new Rom(selectedRom);
                    this.romData.LoadCharacterLevelData();
                    this.romData.LoadMonsterData();
                    justLoadedRom = true;
                    SetFieldsState(true);
                    SetupCharacterDropDownList();
                    cbCharacterList.SelectedIndex = 0;
                    cbLevelList.SelectedIndex = 0;
                    previousCharacterIndex = 0;
                    previousLevelIndex = 0;
                    justLoadedRom = false;
                }
            }
            catch (Exception x)
            {
                MessageBox.Show("Error loading rom: " + x.ToString(), Application.ProductName);
            }
        }

        /// <summary>
        /// saveToolStripMenuItem Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveCharacterLevelFields();
            this.romData.SaveCharacterLevelData(selectedRom);
            MessageBox.Show("Saved!", Application.ProductName);
        }

        #endregion

        #region Methods
                
        /// <summary>
        /// SetupLevelDropDownList
        /// </summary>
        private void SetupLevelDropDownList()
        {
            List<int> levelList = new List<int>();

            for (int i = 2; i < 100; i++)
            {
                levelList.Add(i);
            }

            cbLevelList.DataSource = levelList;
        }

        /// <summary>
        /// SetupCharacterDropDownList
        /// </summary>
        private void SetupCharacterDropDownList()
        {
            isRetranslatedRom = false;

            characterSets = Character.LoadCharacterList();
            cbCharacterList.ValueMember = "CharacterValue";
            cbCharacterList.DataSource = characterSets;

            if (this.romData.monsterName.Length > 0)
            {
                if (enemyNames != null && enemyNames[0] != this.romData.monsterName[0])
                {
                    isRetranslatedRom = true;
                }
            }

            if (isRetranslatedRom)
            {
                cbCharacterList.DisplayMember = "CharacterDescriptionRetranslated";
            }
            else
            {
                cbCharacterList.DisplayMember = "CharacterDescriptionOriginal";
            }
        }

        /// <summary>
        /// SetupCharacterLevelFields
        /// </summary>
        private void SetupCharacterLevelFields()
        {
            int characterIndex = this.cbCharacterList.SelectedIndex;
            int levelIndex = this.cbLevelList.SelectedIndex;
                        
            numCharacterHp.Value = this.romData.characterMaxHPGain[characterIndex][levelIndex];
            numCharacterMp.Value = this.romData.characterMaxMPGain[characterIndex][levelIndex];
            numCharacterAtk.Value = this.romData.characterAtkGain[characterIndex][levelIndex];
            numCharacterDef.Value = this.romData.characterDefGain[characterIndex][levelIndex];
            numCharacterAgi.Value = this.romData.characterAgiGain[characterIndex][levelIndex];
            numCharacterLuck.Value = this.romData.characterLuckGain[characterIndex][levelIndex];
            numCharacterWisdom.Value = this.romData.characterWisdomGain[characterIndex][levelIndex];
            numCharacterGuts.Value = this.romData.characterGutsGain[characterIndex];
            numCharacterExp.Value = this.romData.characterExpNeeded[characterIndex][levelIndex];
        }

        /// <summary>
        /// SaveCharacterLevelFields
        /// </summary>
        private void SaveCharacterLevelFields()
        {
            int characterIndex = previousCharacterIndex;
            int characterExpIndex = characterIndex;
            int levelIndex = previousLevelIndex;

            if (characterIndex != -1 && levelIndex != -1)
            {
                this.romData.characterMaxHPGain[characterIndex][levelIndex] = Convert.ToInt32(numCharacterHp.Value);
                this.romData.characterMaxMPGain[characterIndex][levelIndex] = Convert.ToInt32(numCharacterMp.Value);

                this.romData.characterAtkGain[characterIndex][levelIndex] = Convert.ToInt32(numCharacterAtk.Value);
                this.romData.characterDefGain[characterIndex][levelIndex] = Convert.ToInt32(numCharacterDef.Value);
                this.romData.characterAgiGain[characterIndex][levelIndex] = Convert.ToInt32(numCharacterAgi.Value);
                this.romData.characterLuckGain[characterIndex][levelIndex] = Convert.ToInt32(numCharacterLuck.Value);
                this.romData.characterWisdomGain[characterIndex][levelIndex] = Convert.ToInt32(numCharacterWisdom.Value);
                this.romData.characterGutsGain[characterIndex] = Convert.ToInt32(numCharacterGuts.Value);
                this.romData.characterExpNeeded[characterExpIndex][levelIndex] = Convert.ToInt32(numCharacterExp.Value);
            }
        }

        /// <summary>
        /// SetFieldsState
        /// </summary>
        /// <param name="value"></param>
        private void SetFieldsState(bool value)
        {
            this.cbCharacterList.Enabled = value;
            this.cbLevelList.Enabled = value;
            this.numCharacterHp.Enabled = value;
            this.numCharacterMp.Enabled = value;
            this.numCharacterAtk.Enabled = value;
            this.numCharacterDef.Enabled = value;
            this.numCharacterAgi.Enabled = value;
            this.numCharacterLuck.Enabled = value;
            this.numCharacterWisdom.Enabled = value;
            this.numCharacterGuts.Enabled = value;
            this.numCharacterExp.Enabled = value;
            this.btnCharacterMaxStats.Enabled = value;
            this.btnCharacterMaxStatsAll.Enabled = value;
            this.saveToolStripMenuItem.Enabled = value;
        }

        #endregion
    }
}