﻿using Common.Classes;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace BOF2MonsterEditor
{
    public partial class MainForm : Form
    {
        #region Fields

        private Rom romData;
        private string selectedRom;
        private bool justLoadedRom;            // prevents the program to do some things when it loads rom data
        private int previousMonsterIndex;
        private bool isRetranslatedRom;
        private string[] enemyNames;
        
        #endregion

        #region Constructor

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        #region Event Handlers
        
        /// <summary>
        /// aboutToolStripMenuItem_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox aboutBox = new AboutBox();
            aboutBox.ShowDialog();
        }

        /// <summary>
        /// btnMonsterTwiceExp Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMonsterTwiceExp_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("You're about to double the experience given by EVERY ENEMY in the game. Proceed?",
                Application.ProductName,
                MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                this.romData.DoubleExperienceForAllMonsters();
                MessageBox.Show("Done!", Application.ProductName);
                SetupMonsterFields();
            }
        }

        /// <summary>
        /// btnMonsterTwiceZenny Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMonsterTwiceZenny_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("You're about to double the zenny given by EVERY ENEMY in the game. Proceed?",
                Application.ProductName,
                MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                this.romData.DoubleZennyForAllMonsters();
                MessageBox.Show("Done!", Application.ProductName);
                SetupMonsterFields();
            }
        }

        /// <summary>
        /// btnMonsterDropRateAll Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMonsterDropRateAll_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("You're about to set the drop rate given by EVERY ENEMY in the game. Proceed?", 
                Application.ProductName,
                MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                this.romData.SetDropRateForAllMonsters((int)this.cbMonsterDropRateAll.SelectedValue);
                MessageBox.Show("Done!", Application.ProductName);
                SetupMonsterFields();
            }
        }

        /// <summary>
        /// cbMonsterList SelectedIndexChanged
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbMonsterList_SelectedIndexChanged(object sender, EventArgs e)
        {
            int monsterIndex = this.cbMonsterList.SelectedIndex;
            if (!justLoadedRom)
            {
                SaveMonsterFields();
            }

            SetupMonsterFields();

            previousMonsterIndex = this.cbMonsterList.SelectedIndex;
        }

        /// <summary>
        /// exitToolStripMenuItem Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        /// <summary>
        /// linkInformation LinkClicked
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkInformation_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.gamefaqs.com/snes/563530-breath-of-fire-ii/faqs/5436");
        }

        /// <summary>
        /// MainForm Load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            SetFieldsState(false);
            enemyNames = Monster.LoadEnemyNameFile();
            SetupItemSetDropDownList();
            SetupDropRateDropDownList(cbMonsterDropRate);
            SetupDropRateDropDownList(cbMonsterDropRateAll);
            cbMonsterDropRateAll.SelectedValue = 7;
        }

        /// <summary>
        /// openToolStripMenuItem Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                System.Windows.Forms.DialogResult result = this.openFileOpenRom.ShowDialog(this);
                if (result.Equals(System.Windows.Forms.DialogResult.OK))
                {
                    selectedRom = this.openFileOpenRom.FileName;
                    this.Text = "BoF2 Monster Editor - " + selectedRom;
                    this.romData = new Rom(selectedRom);
                    this.romData.LoadMonsterData();
                    justLoadedRom = true;
                    SetFieldsState(true);
                    SetupMonsterDropDownList();
                    cbMonsterList.SelectedIndex = 0;
                    previousMonsterIndex = 0;
                    justLoadedRom = false;
                }
            }
            catch (Exception x)
            {
                MessageBox.Show("Error loading rom: " + x.ToString(), Application.ProductName);
            }
        }

        /// <summary>
        /// saveToolStripMenuItem Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveMonsterFields();
            this.romData.SaveMonsterData(selectedRom);
            MessageBox.Show("Saved!", Application.ProductName);
        }

        #endregion

        #region Methods

        /// <summary>
        /// SetupDropRateDropDownList
        /// </summary>
        private void SetupDropRateDropDownList(ComboBox cbDropRate)
        {
            List<DropRate> dropRates = DropRate.LoadDropRateList();

            cbDropRate.ValueMember = "RateValue";
            cbDropRate.DisplayMember = "RateDescription";
            cbDropRate.DataSource = dropRates;
        }

        /// <summary>
        /// SetupItemSetDropDownList
        /// </summary>
        private void SetupItemSetDropDownList()
        {
            List<ItemSet> itemSets = ItemSet.LoadItemSetList();

            cbMonsterItemSet.ValueMember = "SetValue";
            cbMonsterItemSet.DataSource = itemSets;
        }

        /// <summary>
        /// SetupMonsterDropDownList
        /// </summary>
        private void SetupMonsterDropDownList()
        {
            isRetranslatedRom = false;
            cbMonsterList.Items.Clear();

            if(this.romData.monsterName.Length > 0)
            {
                if (enemyNames != null && enemyNames[0] != this.romData.monsterName[0])
                {
                    isRetranslatedRom = true;
                }

                for (var i = 0; i < this.romData.monsterName.Length; i++)
                {
                    cbMonsterList.Items.Add(new Monster
                                            {
                                                MonsterRomAddress = this.romData.monsterNameOffset[i],
                                                NameOriginal = this.romData.monsterName[i],
                                                NameRetranslated = enemyNames[i]
                                            }
                                           );
                }

                cbMonsterList.ValueMember = "MonsterRomAddress";
            }

            if (isRetranslatedRom)
            {
                cbMonsterList.DisplayMember = "NameRetranslated";
                cbMonsterItemSet.DisplayMember = "SetDescriptionRetranslated";
            }
            else
            {
                cbMonsterList.DisplayMember = "NameOriginal";
                cbMonsterItemSet.DisplayMember = "SetDescriptionOriginal";
            }
        }

        /// <summary>
        /// SetupMonsterFields
        /// </summary>
        private void SetupMonsterFields()
        {
            int monsterIndex = this.cbMonsterList.SelectedIndex;
            System.Diagnostics.Debug.WriteLine(monsterIndex);

            numMonsterHp.Value = this.romData.monsterMaxHP[monsterIndex];
            numMonsterMp.Value = this.romData.monsterMaxMP[monsterIndex];
            numMonsterAtk.Value = this.romData.monsterAtk[monsterIndex];
            numMonsterDef.Value = this.romData.monsterDef[monsterIndex];
            numMonsterAgi.Value = this.romData.monsterAgi[monsterIndex];
            numMonsterLuk.Value = this.romData.monsterLuk[monsterIndex];
            numMonsterExp.Value = this.romData.monsterExp[monsterIndex];
            numMonsterZenny.Value = this.romData.monsterZenny[monsterIndex];
            cbMonsterDropRate.SelectedValue = this.romData.monsterDropRate[monsterIndex];
            cbMonsterItemSet.SelectedValue = this.romData.monsterItemSet[monsterIndex];

            txtMonsterBytes.Text = this.romData.GetMonsterByteData(monsterIndex);
        }

        /// <summary>
        /// SaveMonsterFields
        /// </summary>
        private void SaveMonsterFields()
        {
            int monsterIndex = previousMonsterIndex;

            this.romData.monsterMaxHP[monsterIndex] = Convert.ToInt32(numMonsterHp.Value);
            this.romData.monsterMaxMP[monsterIndex] = Convert.ToInt32(numMonsterMp.Value);

            this.romData.monsterAtk[monsterIndex] = Convert.ToInt32(numMonsterAtk.Value);
            this.romData.monsterDef[monsterIndex] = Convert.ToInt32(numMonsterDef.Value);
            this.romData.monsterAgi[monsterIndex] = Convert.ToInt32(numMonsterAgi.Value);
            this.romData.monsterLuk[monsterIndex] = Convert.ToInt32(numMonsterLuk.Value);
            this.romData.monsterExp[monsterIndex] = Convert.ToInt32(numMonsterExp.Value);
            this.romData.monsterZenny[monsterIndex] = Convert.ToInt32(numMonsterZenny.Value);
            this.romData.monsterDropRate[monsterIndex] = Convert.ToInt32(cbMonsterDropRate.SelectedValue);
            this.romData.monsterItemSet[monsterIndex] = Convert.ToInt32(cbMonsterItemSet.SelectedValue);

        }

        /// <summary>
        /// SetFieldsState
        /// </summary>
        /// <param name="value"></param>
        private void SetFieldsState(bool value)
        {
            this.cbMonsterList.Enabled = value;
            this.numMonsterHp.Enabled = value;
            this.numMonsterExp.Enabled = value;
            this.numMonsterAtk.Enabled = value;
            this.numMonsterDef.Enabled = value;
            this.numMonsterAgi.Enabled = value;
            this.numMonsterLuk.Enabled = value;
            this.numMonsterMp.Enabled = value;
            this.numMonsterZenny.Enabled = value;
            this.cbMonsterDropRate.Enabled = value;
            this.cbMonsterDropRateAll.Enabled = value;
            this.cbMonsterItemSet.Enabled = value;
            this.btnMonsterTwiceExp.Enabled = value;
            this.btnMonsterTwiceZenny.Enabled = value;
            this.btnMonsterDropRateAll.Enabled = value;
            this.saveToolStripMenuItem.Enabled = value;
        }

        #endregion
    }
}