﻿using System.Collections.Generic;

namespace Common.Classes
{
    /// <summary>
    /// Class containing fields for each Item Set that is shown in the Item Set Drop-Down Lists.
    /// </summary>
    public class Character
    {
        #region Properties

        /// <summary>
        /// Character Value
        /// </summary>
        public int CharacterValue { get; set; } 

        /// <summary>
        /// Character Description (From original English ROM)
        /// </summary>
        public string CharacterDescriptionOriginal { get; set; }

        /// <summary>
        /// Character Description (To be shown for retranslated English ROM)
        /// </summary>
        public string CharacterDescriptionRetranslated { get; set; }

        #endregion

        #region Constructor

        /// <summary>
        /// Constructor for class Character
        /// </summary>
        /// <param name="characterValue"></param>
        /// <param name="characterDescriptionOriginal"></param>
        /// <param name="characterDescriptionRetranslated"></param>
        public Character(int characterValue, string characterDescriptionOriginal, string characterDescriptionRetranslated = null) 
        {
            CharacterValue = characterValue;
            CharacterDescriptionOriginal = characterDescriptionOriginal;

            if (characterDescriptionRetranslated == null)
            {
                CharacterDescriptionRetranslated = characterDescriptionOriginal;
            }
            else
            {
                CharacterDescriptionRetranslated = characterDescriptionRetranslated;
            }
        }

        #endregion

        #region Public Static Functions

        public static List<Character> LoadCharacterList()
        {
            List<Character> characters = new List<Character>();

            characters.Add(new Character(0, "Ryu"));
            characters.Add(new Character(1, "Bow", "Bosch"));
            characters.Add(new Character(2, "Rand"));
            characters.Add(new Character(3, "Katt", "Lin"));
            characters.Add(new Character(4, "Nina"));
            characters.Add(new Character(5, "Jean", "Tapeta"));
            characters.Add(new Character(6, "Sten"));
            characters.Add(new Character(7, "Spar", "Aspara"));
        
            return characters;
        }               

        #endregion
    }
}
