﻿using System.Collections.Generic;

namespace Common.Classes
{
    /// <summary>
    /// Class containing fields for each Item Set that is shown in the Item Set Drop-Down Lists.
    /// </summary>
    public class ItemSet
    {
        #region Properties

        /// <summary>
        /// Set Value
        /// </summary>
        public int SetValue { get; set; } 

        /// <summary>
        /// Set Description (From original English ROM)
        /// </summary>
        public string SetDescriptionOriginal { get; set; }

        /// <summary>
        /// Set Description (To be shown for retranslated English ROM)
        /// </summary>
        public string SetDescriptionRetranslated { get; set; }

        #endregion

        #region Constructor

        /// <summary>
        /// Constructor for class ItemSet
        /// </summary>
        /// <param name="setValue"></param>
        /// <param name="setDescriptionOriginal"></param>
        /// <param name="setDescriptionRetranslated"></param>
        public ItemSet(int setValue, string setDescriptionOriginal, string setDescriptionRetranslated = null) 
        {
            SetValue = setValue;
            SetDescriptionOriginal = setDescriptionOriginal;

            if (setDescriptionRetranslated == null)
            {
                SetDescriptionRetranslated = setDescriptionOriginal;
            }
            else
            {
                SetDescriptionRetranslated = setDescriptionRetranslated;
            }
        }

        #endregion

        #region Public Static Functions

        public static List<ItemSet> LoadItemSetList()
        {
            List<ItemSet> itemSets = new List<ItemSet>();

            itemSets.Add(new ItemSet(0, "Herb", "Medical Herb"));
            itemSets.Add(new ItemSet(1, "Antdt", "Antidote"));
            itemSets.Add(new ItemSet(2, "Herb, HelpBL", "Medical Herb, Vitamin"));
            itemSets.Add(new ItemSet(3, "CureAl", "Panacea"));
            itemSets.Add(new ItemSet(4, "LifePl", "Ammonia"));
            itemSets.Add(new ItemSet(5, "Stamina", "Stamina Up"));
            itemSets.Add(new ItemSet(6, "LuckCndy", "Lucky Charm"));
            itemSets.Add(new ItemSet(7, "PwrFood", "Power Food"));
            itemSets.Add(new ItemSet(8, "Dinker, MisoSoup", "Fast Food, Miso Stew"));
            itemSets.Add(new ItemSet(9, "EarthAR, LoveBR", "Earth Armor, Love Brace"));
            itemSets.Add(new ItemSet(10, "Shaker", "Dynamite"));
            itemSets.Add(new ItemSet(11, "LopOffWP, ElmoreSD", "Headhunter, Elmore Sword"));
            itemSets.Add(new ItemSet(12, "IceChunk", "Icicle"));
            itemSets.Add(new ItemSet(13, "Charm, ShinyBR", "Amulet, Light Brace"));
            itemSets.Add(new ItemSet(14, "LoveBR, LifeBR", "Love Brace, Life Brace"));
            itemSets.Add(new ItemSet(15, "GutsBT, HeroBT", "Guts Belt, Courage Belt"));
            itemSets.Add(new ItemSet(16, "MoonDrop", "Moon Drop"));
            itemSets.Add(new ItemSet(17, "WFruit, WiseBL", "Wisdom Seed, Wisdom Fruit"));
            itemSets.Add(new ItemSet(18, "StoneAR, HeroAR", "Stone Armor, Tyrant Armor"));
            itemSets.Add(new ItemSet(19, "GutsBT", "Guts Belt"));
            itemSets.Add(new ItemSet(20, "DreamBR", "Dream Brace"));
            itemSets.Add(new ItemSet(21, "D.Earing, WorthRG", "Dream Jewel, Death Ring"));
            itemSets.Add(new ItemSet(22, "SkullBR", "Skull Brace"));
            itemSets.Add(new ItemSet(23, "CloakKN, LopOffWP", "Hawk Claw, Headhunter"));
            itemSets.Add(new ItemSet(24, "FireRock, BraveAR", "Firecracker, Brave Armor"));
            itemSets.Add(new ItemSet(25, "Frizbee", "Thunder Rod"));
            itemSets.Add(new ItemSet(26, "IronSH, StarSH", "Iron Shield, Star Shield"));
            itemSets.Add(new ItemSet(27, "SkullBR, CrsntSD", "Skull Brace, Scimitar"));
            itemSets.Add(new ItemSet(28, "MystryHT", "Mystery Helm"));
            itemSets.Add(new ItemSet(29, "SteelAR", "Iron Band"));
            itemSets.Add(new ItemSet(30, "Herb, SilverDR", "Medical Herb, Silver Knife"));
            itemSets.Add(new ItemSet(31, "CharmSH, SilverAR", "Defender, Silver Armer"));
            itemSets.Add(new ItemSet(32, "Biscuit, SlicerDR", "Hardtack, Slicer"));
            itemSets.Add(new ItemSet(33, "MeowST", "Kitty Staff"));
            itemSets.Add(new ItemSet(34, "Medicate, FireRG", "Ginseng, Fire Ring"));
            itemSets.Add(new ItemSet(35, "ElmoreSD, DemonDR", "Elmore Sword, Diablo Knife"));
            itemSets.Add(new ItemSet(36, "MedusaSH, NoTwinRP", "Ares Shield, Balmung"));
            itemSets.Add(new ItemSet(37, "Extract, TigerSD", "Restorative, Vigor Sword"));
            itemSets.Add(new ItemSet(38, "WiseBL, LopOffWP", "Wisdom Fruit, Headhunter"));
            itemSets.Add(new ItemSet(39, "SacredSH, ArmyGL", "Holy Plate, Tyr's Gloves"));
            itemSets.Add(new ItemSet(40, "IceChunk, FrozenAR", "Icicle, Ice Armor"));
            itemSets.Add(new ItemSet(41, "Bandana", "Bandana"));
            itemSets.Add(new ItemSet(42, "AmberRG, SacredSH", "Amber Ring, Holy Plate"));
            itemSets.Add(new ItemSet(43, "PwrFood, MamothCL", "Power Food, Mamoth Pelt"));
            itemSets.Add(new ItemSet(44, "GuardSH, NinjaML", "Guardian, Ninja Suit"));
            itemSets.Add(new ItemSet(45, "BronzeSD, JahAR", "Bronze Sword, Scale Armor"));
            itemSets.Add(new ItemSet(46, "SilverHT, WisdomRB", "Silver Tiara, Wisdom Robe"));
            itemSets.Add(new ItemSet(47, "StoneAR, EarthAR", "Stone Armor, Earth Armor"));
            itemSets.Add(new ItemSet(48, "Stamina, ArmyGL", "Stamina Up, Tyr's Gloves"));
            itemSets.Add(new ItemSet(49, "KmikzeBL, DemonDR", "Void Sphere, Diablo Knife"));
            itemSets.Add(new ItemSet(50, "IronML,SlashGL", "Iron Plate, Slash Gloves"));
            itemSets.Add(new ItemSet(51, "Gold", "Coin"));
            itemSets.Add(new ItemSet(52, "ArmyGL, 9-TailWP", "Tyr's Gloves, Cat 'o Nine"));
            itemSets.Add(new ItemSet(53, "BreathAR, BlndedAR", "Breath Armor, Alloy Armor"));
            itemSets.Add(new ItemSet(54, "ThornWP, GlassDR", "Thorn Whip, Newbie Sword"));
            itemSets.Add(new ItemSet(55, "WhiteSH, SilverAR", "Shine Shield, Silver Armor"));
            itemSets.Add(new ItemSet(56, "HolySH", "Soul Shield"));
            itemSets.Add(new ItemSet(57, "Van.Ext, ElemntAR", "Vitalizer, Gale Band"));
            itemSets.Add(new ItemSet(58, "BronzeHT, IronSH", "Bronze Helm, Iron Shield"));
            itemSets.Add(new ItemSet(59, "WizardRG, FireRG", "Warlock Ring, Fire Ring"));
            itemSets.Add(new ItemSet(60, "SokletAR, CrmsnCL", "Circlet, Crimson Cape"));
            itemSets.Add(new ItemSet(61, "Extract, Van.Ext", "Restorative, Vitalizer"));
            itemSets.Add(new ItemSet(62, "Charm", "Amulet"));
        
            return itemSets;
        }               

        #endregion
    }
}
