﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;

namespace Common.Classes
{
    /// <summary>
    /// Class containing fields for each Monster that is shown in the Monster Drop-Down List.
    /// </summary>
    public class Monster
    {
        #region Properties

        /// <summary>
        /// Monster Rom Address 
        /// </summary>
        public string MonsterRomAddress { get; set; } 

        /// <summary>
        /// Monster Name (From original English ROM)
        /// </summary>
        public string NameOriginal { get; set; }

        /// <summary>
        /// Monster Name (To be shown for retranslated ROM - values read from EnemyNames.txt file in Data sub-folder)
        /// </summary>
        public string NameRetranslated { get; set; }

        #endregion

        #region Public Static Functions

        /// <summary>
        /// LoadEnemyNameFile
        /// </summary>
        public static string[] LoadEnemyNameFile()
        {
            Stream enemyStream = Assembly.GetExecutingAssembly().GetManifestResourceStream("Common.Data.EnemyNames.txt");
            List<string> enemyNameList = new List<string>();

            using (StreamReader reader = new StreamReader(enemyStream))
            {
                while (reader.EndOfStream == false)
                {
                    enemyNameList.Add(reader.ReadLine());
                }
            }

            return enemyNameList.ToArray();
        }

        /// <summary>
        /// LoadMonsterFormationDataFile
        /// </summary>
        public static string LoadMonsterFormationDataFile()
        {
            Stream monsterFormationStream = Assembly.GetExecutingAssembly().GetManifestResourceStream("Common.Data.MonsterFormationData.txt");
            List<string> monsterFormationDataList = new List<string>();

            using (StreamReader reader = new StreamReader(monsterFormationStream))
            {
                while (reader.EndOfStream == false)
                {
                    monsterFormationDataList.Add(reader.ReadLine());
                }
            }

            string monsterFormationDataArray = string.Empty;

            foreach (string s in monsterFormationDataList)
            {
                monsterFormationDataArray += s.Replace(" ", string.Empty);
            }

            return monsterFormationDataArray;
        }       

        #endregion
    }
}