﻿using System;
using System.IO;
using System.Text;

namespace Common.Classes
{
    public class Rom
    {
        #region Fields

        #region Monster Fields

        public static int maxMonsters = 200;

        public string[] monsterName = new string[maxMonsters];
        public string[] monsterNameOffset = new string[maxMonsters];
        public char[] monsterUnknown1 = new char[maxMonsters];
        public int[] monsterMaxHP = new int[maxMonsters];
        public int[] monsterMaxMP = new int[maxMonsters];
        public int[] monsterAtk = new int[maxMonsters];
        public int[] monsterDef = new int[maxMonsters];
        public int[] monsterAgi = new int[maxMonsters];
        public int[] monsterLuk = new int[maxMonsters];
        public int[] monsterExp = new int[maxMonsters];
        public int[] monsterZenny = new int[maxMonsters];
        public int[] monsterDropRate = new int[maxMonsters];
        public int[] monsterItemSet = new int[maxMonsters];

        #endregion

        #region Character Level Gain Fields

        public static int maxCharacters = 8;
        public static int maxLevels = 98;

        public string[] characterName = new string[maxCharacters];
        public int[][] characterMaxHPGain = new int[maxCharacters][];
        public int[][] characterMaxMPGain = new int[maxCharacters][];
        public int[][] characterAtkGain = new int[maxCharacters][];
        public int[][] characterDefGain = new int[maxCharacters][];
        public int[][] characterUnknownGain = new int[maxCharacters][];
        public int[][] characterAgiGain = new int[maxCharacters][];
        public int[][] characterLuckGain = new int[maxCharacters][];
        public int[][] characterWisdomGain = new int[maxCharacters][];
        public int[][] characterExpTotal = new int[maxCharacters][];
        public int[][] characterExpNeeded = new int[maxCharacters][];
        public int[] characterGutsGain = new int[maxCharacters];
        
        #endregion

        private byte[] Data;

        #endregion

        #region Constructor

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="path"></param>
        public Rom(string path)
        {
            this.Data = File.ReadAllBytes(path);
        }

        #endregion

        #region Public Methods

        #region ROM Byte Data Functions

        /// <summary>
        /// Subtract Data
        /// </summary>
        /// <param name="offset"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        public byte[] SubtractData(string offset, int length)
        {
            int index = int.Parse(offset, System.Globalization.NumberStyles.HexNumber);         
            byte[] result = new byte[length];
            Array.Copy(Data, index, result, 0, length);
            return result;
        }

        /// <summary>
        /// Read Int Data
        /// </summary>
        /// <param name="offset"></param>
        /// <param name="length"></param>
        /// <param name="littleEndian"></param>
        /// <returns></returns>
        public int ReadIntData(string offset, int length, bool littleEndian)
        {
            byte[] buffer = new byte[length];

            buffer = SubtractData(offset, length);
            System.Text.StringBuilder hexString = new System.Text.StringBuilder(length * 2);

            for (int index = 0; index < length; index++)
            {
                byte byteValue = buffer[index];
                string hexValue = byteValue.ToString("X");
                if (hexValue.Length < 2) { hexValue = "0" + hexValue; }
                if (!littleEndian)
                {
                    hexString.Append(hexValue);
                }
                else
                {
                    hexString.Insert(0, hexValue);
                }
            }

            return (int.Parse(hexString.ToString(), System.Globalization.NumberStyles.HexNumber));
        }

        /// <summary>
        /// Read Byte Data 
        /// </summary>
        /// <param name="offset"></param>
        /// <param name="length"></param>
        /// <param name="littleEndian"></param>
        /// <returns></returns>
        public string ReadByteData(string offset, int length, bool littleEndian)
        {
            byte[] buffer = new byte[length];

            buffer = SubtractData(offset, length);
            System.Text.StringBuilder hexString = new System.Text.StringBuilder(length * 2);

            for (int index = 0; index < length; index++)
            {
                byte byteValue = buffer[index];
                string hexValue = byteValue.ToString("X");
                if (hexValue.Length < 2) { hexValue = "0" + hexValue; }
                if (!littleEndian)
                {
                    hexString.Append(hexValue);
                }
                else
                {
                    hexString.Insert(0, hexValue);
                }
            }
            return (hexString.ToString());
        }

        /// <summary>
        /// Write Byte Data
        /// </summary>
        /// <param name="value"></param>
        /// <param name="offset"></param>
        /// <param name="length"></param>
        /// <param name="littleEndian"></param>        
        public void WriteByteData(string value, string offset, int length, bool littleEndian)
        {
            length *= 2;
            String hexString = value;
            int index = int.Parse(offset, System.Globalization.NumberStyles.HexNumber);
            
            if (!littleEndian)
            {
                for (int i = 0; i < length; i += 2)
                {
                    String substring = hexString.Substring(i, 2);
                    byte b = Convert.ToByte(substring, 16);
                    Data[index + (i / 2)] = b;
                }
            }
            else
            {
                StringBuilder sb = new StringBuilder("0000");
                for (int i = 0; i < length; i += 2)
                {
                    String substring = hexString.Substring(i, 2);
                    byte b = Convert.ToByte(substring, 16);
                    Data[index + ((length - i - 2) / 2)] = b;
                    sb[((length - i - 2) / 2)] = substring[0];
                    sb[((length - i - 2) / 2) + 1] = substring[1];

                }
            }
        }

        /// <summary>
        /// Write Integer Data
        /// </summary>
        /// <param name="value"></param>
        /// <param name="offset"></param>
        /// <param name="length"></param>
        /// <param name="littleEndian"></param>
        public void WriteIntData(int value, string offset, int length, bool littleEndian)
        {
            length *= 2;

            String hexString = value.ToString("X" + (length).ToString());

            int index = int.Parse(offset, System.Globalization.NumberStyles.HexNumber);

            if (!littleEndian)
            {
                for (int i = 0; i < length; i += 2)
                {
                    String substring = hexString.Substring(i, 2);
                    byte b = Convert.ToByte(substring, 16);
                    Data[index + (i / 2)] = b;
                }
            }
            else
            {
                StringBuilder sb = new StringBuilder("0000");
                for (int i = 0; i < length; i += 2)
                {
                    String substring = hexString.Substring(i, 2);
                    byte b = Convert.ToByte(substring, 16);
                    Data[index + ((length - i - 2) / 2)] = b;
                    sb[((length - i - 2) / 2)] = substring[0];
                    sb[((length - i - 2) / 2) + 1] = substring[1];

                }
            }
        }

        /// <summary>
        /// Add 2 Hex Values Together
        /// </summary>
        /// <param name="value1"></param>
        /// <param name="value2"></param>
        /// <returns></returns>
        public string AddHexValues(string value1, string value2)
        {
            string r;
            int v1 = int.Parse(value1, System.Globalization.NumberStyles.HexNumber);
            int v2 = int.Parse(value2, System.Globalization.NumberStyles.HexNumber);
            r = (v1 + v2).ToString("X");
            return r;
        }

        /// <summary>
        /// Subtract Hex Value 2 from 1
        /// </summary>
        /// <param name="value1"></param>
        /// <param name="value2"></param>
        /// <returns></returns>
        public string SubtractHexValues(string value1, string value2)
        {
            string r;
            int v1 = int.Parse(value1, System.Globalization.NumberStyles.HexNumber);
            int v2 = int.Parse(value2, System.Globalization.NumberStyles.HexNumber);
            r = (v1 - v2).ToString("X");
            return r;
        }

        /// <summary>
        /// Multiply Hex Value by num times
        /// </summary>
        /// <param name="value"></param>
        /// <param name="num"></param>
        /// <returns></returns>
        public string MultiplyHexValue(string value, int num)
        {
            string r;
            int i, total = 0;
            for (i = 0; i < num; i++)
            {
                total += int.Parse(value, System.Globalization.NumberStyles.HexNumber);
            }
            r = total.ToString("X");
            return r;
        }

        /// <summary>
        /// Read Hex Data
        /// </summary>
        /// <param name="offset"></param>
        /// <returns></returns>
        public string ReadHexData(string offset)
        {
            byte[] buffer = new byte[1];

            buffer = SubtractData(offset, 1);

            byte byteValue = buffer[0];
            return (byteValue.ToString("X2"));
        }

        /// <summary>
        /// Read String Data
        /// </summary>
        /// <param name="offset"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        public string ReadStringData(string offset, int length)
        {
            byte[] buffer = new byte[length];
            buffer = SubtractData(offset, length);

            char[] chars = new char[length];
            int index = 0;
            while (index < length)
            {
                chars[index] = (char)buffer[index];
                index++;
            }

            return (new String(chars, 0, index).Trim());
        }

        /// <summary>
        /// Write String Data
        /// </summary>
        /// <param name="value"></param>
        /// <param name="offset"></param>
        /// <param name="length"></param>
        public void WriteStringData(string value, string offset, int length)
        {
            int index = int.Parse(offset, System.Globalization.NumberStyles.HexNumber);
            char[] chars = value.ToCharArray();
            int length2 = chars.Length;
            int i = 0;

            for (; i < length2; i++)
            {
                Data[index + i] = (byte)chars[i];
            }
            
            char space = ' ';
            
            for (; i < length; i++)
            {
                Data[index + i] = (byte)space;
            }
        }

        #endregion

        #region Monster Functions

        /// <summary>
        /// Load Monster Data
        /// </summary>
        public void LoadMonsterData()
        {
            string startOffset = "059000";
            string offset;
            int dataSize;
            string dropRateData;

            for (var i = 0; i < maxMonsters; i += 1)
            {
                offset = AddHexValues(startOffset, this.MultiplyHexValue("20", i));    // 20 is 32 in hex

                dataSize = 8;
                monsterName[i] = ReadStringData(offset, dataSize);
                monsterNameOffset[i] = offset;
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 2;
                monsterMaxHP[i] = ReadIntData(offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 2;
                monsterMaxMP[i] = ReadIntData(offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 1;
                monsterLuk[i] = ReadIntData(offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 2;
                monsterAtk[i] = ReadIntData(offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());
                monsterDef[i] = ReadIntData(offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());
                monsterAgi[i] = ReadIntData(offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 1;
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 2;
                monsterExp[i] = ReadIntData(offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());
                monsterZenny[i] = ReadIntData(offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                // itemset
                dataSize = 1;
                monsterItemSet[i] = ReadIntData(offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                // unknown byte
                dataSize = 1;
                offset = AddHexValues(offset, dataSize.ToString());

                // drop rate!
                dataSize = 1;
                dropRateData = ReadByteData(offset, dataSize, true);
                monsterDropRate[i] = (int)Char.GetNumericValue(dropRateData[0]);
                monsterUnknown1[i] = dropRateData[1];
                offset = AddHexValues(offset, dataSize.ToString());

                // unknown bytes
                dataSize = 5; // 7;
                offset = AddHexValues(offset, dataSize.ToString());
            }
        }

        /// <summary>
        /// Save Monster Data
        /// </summary>
        /// <param name="ROMname"></param>
        public void SaveMonsterData(string ROMname)
        {
            string startOffset = "059000";
            string offset;
            int dataSize;
            string dropRateData;

            for (var i = 0; i < maxMonsters; i += 1)
            {
                offset = AddHexValues(startOffset, this.MultiplyHexValue("20", i));    // 20 is 32 in hex

                dataSize = 8;
                // not saving monster names
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 2;
                WriteIntData(monsterMaxHP[i], offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());
                WriteIntData(monsterMaxMP[i], offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 1;
                WriteIntData(monsterLuk[i], offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 2;
                WriteIntData(monsterAtk[i], offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());
                WriteIntData(monsterDef[i], offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());
                WriteIntData(monsterAgi[i], offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 1;
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 2;
                WriteIntData(monsterExp[i], offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());
                WriteIntData(monsterZenny[i], offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                // save item set
                dataSize = 1;
                WriteIntData(monsterItemSet[i], offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 1;
                // not saving unknown byte
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 1;
                // save drop item rate byte (first character of byte is the drop rate - keep 2nd character from before)
                dropRateData = String.Concat(monsterDropRate[i], monsterUnknown1[i]);
                WriteByteData(dropRateData, offset, dataSize, true);
                offset = AddHexValues(offset, dataSize.ToString());

                dataSize = 5;
                // not saving unknown bytes
                offset = AddHexValues(offset, dataSize.ToString());
            }

            File.WriteAllBytes(ROMname, Data);
        }
                
        /// <summary>
        /// Given a monster index, return a string containing all 32 bytes of monster data.
        /// </summary>
        /// <param name="monsterIndex"></param>
        /// <returns></returns>
        public string GetMonsterByteData(int monsterIndex)
        {
            string startOffset = "059000";
            int dataSize = 8;
            string offset, bytes;

            offset = AddHexValues(startOffset, this.MultiplyHexValue("20", monsterIndex));    // 20 is 32 in hex
            offset = AddHexValues(offset, dataSize.ToString());
            bytes = ReadByteData(offset, 24, false);

            return bytes;
        }

        /// <summary>
        /// Double Experience for all monsters, up to maximum of 65535 experience per monster.
        /// </summary>
        public void DoubleExperienceForAllMonsters()
        {
            for (var i = 0; i < maxMonsters; i += 1)
            {
                monsterExp[i] = monsterExp[i] * 2;

                if (monsterExp[i] > 65535)
                {
                    monsterExp[i] = 65535;
                }                
                else if (monsterExp[i] < 0)
                {
                    monsterExp[i] = 0;
                }
            }
        }

        /// <summary>
        /// Double Zenny (Money) for all monsters, up to maximum of 65535 experience per monster. 
        /// </summary>
        public void DoubleZennyForAllMonsters()
        {
            for (var i = 0; i < maxMonsters; i += 1)
            {
                monsterZenny[i] = monsterZenny[i] * 2;
                if (monsterZenny[i] > 65535)
                {
                    monsterZenny[i] = 65535;
                }
                else if (monsterZenny[i] < 0)
                {
                    monsterZenny[i] = 0;
                }
            }
        }

        /// <summary>
        /// Set Drop Rate for all monsters.  Must be between 0 and 7. 
        /// </summary>
        /// <param name="dropRate"></param>
        public void SetDropRateForAllMonsters(int dropRate)
        {
            for (var i = 0; i < maxMonsters; i += 1)
            {
                monsterDropRate[i] = dropRate;

                if (monsterDropRate[i] > 7)
                {
                    monsterDropRate[i] = 7;
                }
                else if (monsterDropRate[i] < 0)
                {
                    monsterDropRate[i] = 0;
                }
            }
        }

        #endregion

        #region Level Stat Gain Functions

        /// <summary>
        /// Load Character Data
        /// </summary>
        public void LoadCharacterLevelData()
        {
            string statGainOffset = "0571B0";
            string gutsGainOffset = "018FE5";
            string experienceTotalOffset = "056880";
            int dataSize;
            string byteData;

            for (var i = 0; i < maxCharacters; i++)
            {
                characterMaxHPGain[i] = new int[maxLevels];
                characterMaxMPGain[i] = new int[maxLevels];
                characterAtkGain[i] = new int[maxLevels];
                characterDefGain[i] = new int[maxLevels];
                characterUnknownGain[i] = new int[maxLevels];
                characterAgiGain[i] = new int[maxLevels];
                characterWisdomGain[i] = new int[maxLevels];
                characterLuckGain[i] = new int[maxLevels];
                characterExpTotal[i] = new int[maxLevels];
                characterExpNeeded[i] = new int[maxLevels];

                for (int j = 0; j < maxLevels; j++)
                {
                    dataSize = 1;
                    byteData = ReadByteData(statGainOffset, dataSize, true);
                    characterMaxHPGain[i][j] = int.Parse(byteData.Substring(0, 1), System.Globalization.NumberStyles.HexNumber);
                    characterMaxMPGain[i][j] = int.Parse(byteData.Substring(1, 1), System.Globalization.NumberStyles.HexNumber);
                    statGainOffset = AddHexValues(statGainOffset, dataSize.ToString());

                    dataSize = 1;
                    byteData = ReadByteData(statGainOffset, dataSize, true);
                    characterAtkGain[i][j] = int.Parse(byteData.Substring(0, 1), System.Globalization.NumberStyles.HexNumber);
                    characterDefGain[i][j] = int.Parse(byteData.Substring(1, 1), System.Globalization.NumberStyles.HexNumber);
                    statGainOffset = AddHexValues(statGainOffset, dataSize.ToString());

                    dataSize = 1;
                    byteData = ReadByteData(statGainOffset, dataSize, true);
                    characterUnknownGain[i][j] = int.Parse(byteData.Substring(0, 1), System.Globalization.NumberStyles.HexNumber);
                    characterAgiGain[i][j] = int.Parse(byteData.Substring(1, 1), System.Globalization.NumberStyles.HexNumber);
                    statGainOffset = AddHexValues(statGainOffset, dataSize.ToString());

                    dataSize = 1;
                    byteData = ReadByteData(statGainOffset, dataSize, true);
                    characterWisdomGain[i][j] = int.Parse(byteData.Substring(0, 1), System.Globalization.NumberStyles.HexNumber);
                    characterLuckGain[i][j] = int.Parse(byteData.Substring(1, 1), System.Globalization.NumberStyles.HexNumber);
                    statGainOffset = AddHexValues(statGainOffset, dataSize.ToString());

                    dataSize = 3;
                    characterExpTotal[i][j] = ReadIntData(experienceTotalOffset, dataSize, true);
                    characterExpNeeded[i][j] = (j == 0 ? characterExpTotal[i][j] : characterExpTotal[i][j] - characterExpTotal[i][j - 1]);
                    experienceTotalOffset = AddHexValues(experienceTotalOffset, dataSize.ToString());
                }

                // Guts gains are the same every level for each character.
                dataSize = 1;
                characterGutsGain[i] = ReadIntData(gutsGainOffset, dataSize, true);
                gutsGainOffset = AddHexValues(gutsGainOffset, dataSize.ToString());                                    
            }
        }

        /// <summary>
        /// Save Character Data
        /// </summary>
        /// <param name="ROMname"></param>
        public void SaveCharacterLevelData(string ROMname)
        {
            string statGainOffset = "0571B0";
            string gutsGainOffset = "018FE5";
            string experienceTotalOffset = "056880";
            string monsterFormationOffset = "057DF0";
            int dataSize;
            string byteData;

            for (var i = 0; i < maxCharacters; i++)
            {
                for (int j = 0; j < maxLevels; j++)
                {
                    dataSize = 1;
                    byteData = String.Concat(characterMaxHPGain[i][j].ToString("X"), characterMaxMPGain[i][j].ToString("X"));
                    WriteByteData(byteData, statGainOffset, dataSize, true);
                    statGainOffset = AddHexValues(statGainOffset, dataSize.ToString());

                    dataSize = 1;
                    byteData = String.Concat(characterAtkGain[i][j].ToString("X"), characterDefGain[i][j].ToString("X"));
                    WriteByteData(byteData, statGainOffset, dataSize, true);
                    statGainOffset = AddHexValues(statGainOffset, dataSize.ToString());

                    dataSize = 1;
                    byteData = String.Concat(characterUnknownGain[i][j].ToString("X"), characterAgiGain[i][j].ToString("X"));
                    WriteByteData(byteData, statGainOffset, dataSize, true);
                    statGainOffset = AddHexValues(statGainOffset, dataSize.ToString());

                    dataSize = 1;
                    byteData = String.Concat(characterWisdomGain[i][j].ToString("X"), characterLuckGain[i][j].ToString("X"));
                    WriteByteData(byteData, statGainOffset, dataSize, true);
                    statGainOffset = AddHexValues(statGainOffset, dataSize.ToString());

                    dataSize = 3;
                    characterExpTotal[i][j] = (j == 0 ? characterExpNeeded[i][j] : characterExpNeeded[i][j] + characterExpTotal[i][j - 1]);
                    WriteIntData(characterExpTotal[i][j], experienceTotalOffset, dataSize, true);
                    experienceTotalOffset = AddHexValues(experienceTotalOffset, dataSize.ToString());
                }

                dataSize = 1;
                WriteIntData(characterGutsGain[i], gutsGainOffset, dataSize, true);
                gutsGainOffset = AddHexValues(gutsGainOffset, dataSize.ToString());                
            }

            // Make sure Monster Formation Data is intact prior to saving the ROM.  
            // This could have been corrupted by version 1.0 of the Level Stat Gain Editor if Bleu's stats were modified.
            string monsterFormationData = Monster.LoadMonsterFormationDataFile();
            WriteByteData(monsterFormationData, monsterFormationOffset, monsterFormationData.Length / 2, false);

            File.WriteAllBytes(ROMname, Data);
        }
                
        /// <summary>
        /// Max Stats for character at specified index.
        /// </summary>
        /// <param name="characterIndex"></param>
        public void MaxStatsForCharacter(int characterIndex)
        {
            for (int j = 0; j < maxLevels; j++)
            {
                characterMaxHPGain[characterIndex][j] = 0x0F;
                characterMaxMPGain[characterIndex][j] = 0x0F;
                characterAtkGain[characterIndex][j] = 0x0F;
                characterDefGain[characterIndex][j] = 0x0F;
                characterWisdomGain[characterIndex][j] = 0x0F;
                characterLuckGain[characterIndex][j] = 0x0F;
            }

            characterGutsGain[characterIndex] = 0x0F;
        }

        /// <summary>
        /// Max Stats for all characters.
        /// </summary>
        public void MaxStatsForAllCharacters()
        {
            for (int i = 0; i < maxCharacters; i++)
            {
                MaxStatsForCharacter(i);
            }
        }

        /// <summary>
        /// Set Experience Needed at all levels for character at specified index to specified value.
        /// </summary>
        /// <param name="characterIndex"></param>
        public void SetExperienceNeededAtAllLevelsForCharacter(int characterIndex, int experienceNeeded)
        {
            for (int j = 0; j < maxLevels; j++)
            {
                characterExpNeeded[characterIndex][j] = experienceNeeded;
            }
        }

        /// <summary>
        /// Set Experience Needed at all levels for all characters to specified value.
        /// </summary>
        /// <param name="characterIndex"></param>
        public void SetExperienceNeededAtAllLevelsForAllCharacters(int experienceNeeded)
        {
            for (int i = 0; i < maxCharacters; i++)
            {
                SetExperienceNeededAtAllLevelsForCharacter(i, experienceNeeded);
            }
        }

        #endregion

        #endregion
    }
}